﻿//
// Copyright 2005, 2006 Elvis Mehmedović.
// http://emehmedovic.com/
//

import xm.corpus.resizableComponent;

class xm.components.FLVDisplayComponent extends resizableComponent {
	
	[IconFile("FLVDisplayComponent.png")]
	
	[Event("resizeVideo")]
	[Event("start")]
	[Event("complete")]
	[Event("progress")]
	[Event("loaded")]
	[Event("change")]
	[Event("opening")]
	[Event("closed")]
	[Event("seeking")]
	[Event("error")]
	[Event("meta")]
	
	[Event("volume")]
	[Event("zoom")]
	
	[Event("playing")]
	[Event("notPlaying")]
	
	[Event("bufferEmpty")]
	[Event("bufferFull")]
	[Event("bufferFlush")]

	// ToobPlayer Events
	[Event("acquiringYouTubeUrl")]
	[Event("acquiredYouTubeUrl")]
	[Event("errorAcquiringYouTubeUrl")]
	[Event("prePlay")]
	[Event("preStop")]
	
	static var rmFitIntoAR:String = "fit into component area / mind aspect ratio";
	static var rmFitInto:String = "fit into component area";
	static var rmFillOut:String = "fill out component area";
	static var rmCenter:String = "center in component area";
	static var rmCenterOrFitInto:String = "center in component area / shrink if necessary";
	static var rmComponentToVideo:String = "resize component to video size";
	
	private var _mask:MovieClip;
	private var _this:FLVDisplayComponent;
	private var _sound:Sound;
	
	var video:Object;
	var nc:NetConnection;
	var ns:NetStream;
	
	private var oldTime:Number;
	
	private var __url:String;
	private var __volume:Number = 100;
	private var __zoom:Number = 100;
	private var __bufferTime:Number = 0.5;
	private var __resizeMode:String = rmFitIntoAR;
	private var __playing:Boolean;
	
	var disablePlayingEvents:Boolean = false;
	
	var preferredWidth:Number = 320;
	var preferredHeight:Number = 240;
	var duration:Number;
	var opening:Boolean;
	var started:Boolean;
	var preloading:Boolean;

	var preloadingBL:Number;
	var preloadingBT:Number;
	
	var callWhenInitialized:Array;
	
	[Inspectable(defaultValue=false)]
	var autoRepeat:Boolean = false;
	
	[Inspectable(defaultValue=0.5)]
	function get bufferTime():Number {
		return __bufferTime;
	}
	
	function set bufferTime(p_btime:Number) {
		__bufferTime = p_btime;
		ns.setBufferTime(p_btime);
	}
	
	[Inspectable(defaultValue=100)]
	function get zoom():Number {
		return __zoom;
	}
	
	function set zoom(p_zoom:Number) {
		
		__zoom = p_zoom;
		
		if (_currentframe == 1) {
			return;
		}
		
		resetResizeMode();
		dispatchEvent({type: "zoom"});
		
	}
	
	[Inspectable(defaultValue=100)]
	function get volume():Number {
		return __volume;
	}
	
	function set volume(p_volume:Number) {
		__volume = p_volume;
		_sound.setVolume(p_volume);
		dispatchEvent({type: "volume"});
	}
	
	private function setPlaying(p_val:Boolean):Void {
		if (playing != p_val) {
			__playing = p_val;
			if (!disablePlayingEvents) {
				dispatchEvent({type: playing ? "playing" : "notPlaying"});
			}
		}
		else {
			__playing = p_val;
		}
	}
	
	function get playing():Boolean {
		return __playing;
	}
		
	private function __onMetaData(p_eobj:Object):Void {
		_this.___onMetaData(p_eobj);
	}
	
	private function ___onMetaData(p_eobj:Object):Void {
		bufferTime = bufferTime;
		if (p_eobj.width) {
			preferredWidth = p_eobj.width;
		}
		if (p_eobj.height) {
			preferredHeight = p_eobj.height;
		}
		if (p_eobj.duration) {
			duration = parseFloat(p_eobj.duration);
		}
		else {
			delete duration;
		}
		resetResizeMode();
		if (playing) {
			ns.pause(false);
		}
		dispatchEvent({type: "meta"});
	}
	
	private function __onStatus(p_eobj:Object):Void {
		_this.___onStatus(p_eobj);
	}
	
	private function ___onStatus(p_eobj:Object):Void {
		if (p_eobj.code == "NetStream.Play.StreamNotFound") {
			setPlaying(false);
			preloading = false;
			preloadingBL = undefined;
			preloadingBT = undefined;
			started = false;
			opening = false;
			dispatchEvent({type: "closed"});
			dispatchEvent({type: "change"});
			dispatchEvent({type: "error"});
		}
		else if (p_eobj.code == "NetStream.Buffer.Empty") {
			dispatchEvent({type: "bufferEmpty"});
		}
		else if (p_eobj.code == "NetStream.Buffer.Full") {
			dispatchEvent({type: "bufferFull"});
		}
		else if (p_eobj.code == "NetStream.Buffer.Flush") {
			dispatchEvent({type: "bufferFlush"});
		}
		else if (p_eobj.code == "NetStream.Play.Start") {
			started = true;
			opening = false;
			dispatchEvent({type: "start"});
			setPlaying(true);
			video.smoothing = true;
		}
		else if (p_eobj.code == "NetStream.Play.Stop") {
			var wasPlaying:Boolean = playing;
			if (wasPlaying) {
				if (autoRepeat) {
					time = 0;
					ns.pause(false);
				}
				else {
					pause(true);
					time = 0;
				}
			}
			dispatchEvent({type: "complete"});
		}
	}

	[Inspectable(defaultValue="fit into component area / mind aspect ratio", enumeration="fit into component area / mind aspect ratio,fit into component area,fill out component area,center in component area,center in component area / shrink if necessary,resize component to video size")]
	function get resizeMode():String {
		return __resizeMode;
	}
	
	function set resizeMode(p_mode:String) {
		
		__resizeMode = p_mode;
		
		if (_currentframe == 1) {
			return;
		}
		
		resetResizeMode();
		
	}
	
	function resetResizeMode(Void):Void {
		
		var pWidth:Number = Math.round(preferredWidth * zoom / 100);
		var pHeight:Number = Math.round(preferredHeight * zoom / 100);
		
		var rm:String = resizeMode;
		
		if (rm == rmCenterOrFitInto) {
			rm = (pWidth > width || pHeight > height) ? rmFitIntoAR : rmCenter;
		}
		
		if (rm == rmCenter) {
			
			video._width = pWidth;
			video._height = pHeight;
			video._x = Math.ceil((width - pWidth)/2);
			video._y = Math.ceil((height - pHeight)/2);
			
		}
		else if (rm == rmFitInto) {
			
			video._width = width;
			video._height = height;
			video._x = 0;
			video._y = 0;
			
		}
		else if (rm == rmComponentToVideo) {
			
			video._width = pWidth;
			video._height = pHeight;
			video._x = 0;
			video._y = 0;
			setSize(pWidth, pHeight);
			
		}
		else {
			
			var cond:Boolean = width/height > pWidth/pHeight;
			if (rm == rmFillOut ? cond : !cond) {
				video._width = width;
				video._height = width * pHeight/pWidth;
				video._y = -Math.round((video._height - height)/2);
				video._x = 0;
			}
			else {
				video._height = height;
				video._width = height * pWidth/pHeight;
				video._x = -Math.round((video._width - width)/2);
				video._y = 0;
			}
			
		}

		dispatchEvent({type: "resizeVideo"});
				
	}
	
	function setSize(p_width:Number, p_height:Number):Void {
		
		super.setSize(p_width, p_height);
		
		if (resizeMode != rmComponentToVideo) {
			resetResizeMode();
		}
		
	}
	
	[Inspectable()]
	var autoPlay:Boolean = true;

	[Inspectable()]
	function get url():String {
		return __url;
	}
	
	private var stuckTime:Number;
	
	function onEnterFrame(Void):Void {
		
		if (preloading) {
			if (preloadingBL != ns.bytesLoaded || preloadingBT != ns.bytesTotal) {
				preloadingBL = ns.bytesLoaded;
				preloadingBT = ns.bytesTotal;
				dispatchEvent({type: "progress", bl: preloadingBL, bt: preloadingBT});
			}
			if (preloadingBL >= preloadingBT && preloadingBT > 0) {
				preloading = false;
				dispatchEvent({type: "loaded"});
			}
		}
		
		if (time != oldTime) {
			oldTime = time;
			stuckTime = getTimer();
			dispatchEvent({type: "change"});
		}
		else {
			if (playing && !preloading) {
				if (getTimer() - stuckTime > 1000) {
					time++;
				}
			}
			else {
				stuckTime = getTimer();
			}
		}
		
	}

	function set url(p_val:String) {

		__url = p_val;
		
		if (_currentframe == 1 || url == "" || url == undefined) {
			return;
		}
		
		stop();

		if (autoPlay) {
			play();
		}

	}
	
	function clear(Void):Void {
		stop();
		video.clear();
	}
	
	function play(Void):Void {
		
		if (url == "" || url == undefined) {
			return;
		}
		
		setPlaying(false);
		
		started = false;
		opening = true;
		
		preloading = true;
		preloadingBL = undefined;
		preloadingBT = undefined;
		
		ns.setBufferTime(0);
		ns.play(url);
		ns.pause(true);
		
		dispatchEvent({type: "opening"});
		
	}
	
	function pause(p_pause:Boolean):Void {
		ns.pause(p_pause);
		if (p_pause == undefined) {
			setPlaying(!playing);
		}
		else if (p_pause) {
			setPlaying(false);
		}
		else {
			setPlaying(true);
		}
	}
	
	function stop(Void):Void {
		
		setPlaying(false);
		
		preloading = false;
		started = false;
		opening = false;
		
		delete duration;
		
		ns.close();
		
		preloadingBL = ns.bytesLoaded;
		preloadingBT = ns.bytesTotal;
		
		dispatchEvent({type: "closed"});
		dispatchEvent({type: "progress", bl: preloadingBL, bt: preloadingBT});
		
	}
	
	function get time():Number {
		return ns.time;
	}
	
	function set time(p_time:Number) {
		ns.seek(p_time);
		dispatchEvent({type: "seeking"});
	}
	
	function FLVDisplayComponent() {
	}

	function init(Void):Void {

		super.init();
		nextFrame();
		
		Aligner.register(_mask, { hStretch: true, vStretch: true });
		setMask(_mask);
			
		// video init
		
			nc = new NetConnection();
			nc.connect(null);
			
			ns = new NetStream(nc);
			video.smoothing = true;
			video.attachVideo(ns);
			
			ns["_this"] = this;
			ns["onMetaData"] = __onMetaData;
			ns.onStatus = __onStatus;
			
			_mask.attachAudio(ns);
			_sound = new Sound(_mask);
			
			for (var i:Number = 0; i < callWhenInitialized.length; i++) {
				var o:Object = callWhenInitialized[i];
				o.f.apply(o.o);
			}
			
			
		// properties

			bufferTime = bufferTime;
			url = url;
			volume = volume;
			resizeMode = resizeMode;

	}

}