﻿//
// Copyright 2005 Elvis Mehmedovic.
// http://emehmedovic.com/
//

import xm.corpus.dispatch;
import xm.components.MenuComponent;

class xm.components.MenuInteractionController {
	
	[Event("gotFocus")]
	[Event("lostFocus")]
	[Event("rollOver")]
	[Event("rollOut")]
	[Event("itemSelect")]
	
	var activateReenables:Boolean = false;
	var justCreated:Boolean = true;
	
	private var __hasFocus:Boolean = false;
	private var __enabled:Boolean = true;
	
	var isMouseOver:Boolean = false;
	var isMouseDown:Boolean;
	
	var menus:Array;
	var lastItems:Array;
	
	var lastMouseOver:Number;
	var lastSpawned:Number;
	
	var activatorKey:Number = 120 // F9
	var disableKeyboard:Boolean;
	
	function get enabled():Boolean {
		return __enabled;
	}
	
	function set enabled(p_val:Boolean) {
		__enabled = p_val;
	}
	
	function get hasFocus():Boolean {
		return __hasFocus;
	}
	
	function set hasFocus(p_val:Boolean) {
		if (p_val != __hasFocus) {
			if (__hasFocus = p_val) {
				dispatchEvent({type: "gotFocus"});
			}
			else {
				dispatchEvent({type: "lostFocus"});
			}
		}
	}
	
	function setState(p_menu:Number, p_item:Number, p_state:Number):Void {
		if (p_item != lastItems[p_menu]) {
			if (menus[p_menu].getState(lastItems[p_menu]) != 0) {
				menus[p_menu].refreshItem(lastItems[p_menu], 0);
			}
			lastItems[p_menu] = p_item;
		}
		if (menus[p_menu].getState(p_item) != p_state) {
			menus[p_menu].refreshItem(p_item, p_state);
		}
	}
	
	function checkIfMouseOver(Void):Void {
		var isOver:Boolean =  menus[0].hitTest(_root._xmouse, _root._ymouse, true);
		if (isOver != isMouseOver) {
			if (isMouseOver = isOver) {
				dispatchEvent({type: "rollOver", menu: menus[0]});
			}
			else {
				dispatchEvent({type: "rollOut"});
			}
		}
	}
	
	function onMouseMove(Void):Object {
		
		if (!enabled) {
			return;
		}
		
		for (var i:Number = menus.length-1; i >= 0; i--) {
			
			var menu:MenuComponent = menus[i];
			if (!menu.hitTest(_root._xmouse, _root._ymouse, true)) {
				if (menu.expanded) {
					if (menus[0].hitTest(_root._xmouse, _root._ymouse, true)) {
						if (i != 0 || menu.menuProperties.forcePulldownBehavior) {
							menu.delayedSpawn(null);
						}
						else {
							menu.removeSpawn();
						}
					}
				}
				if (i == 0) {
					if (isMouseOver) {
						isMouseOver = false;
						dispatchEvent({type: "rollOut"});
						
						// onRolledOut safety net
						if (menus[menus.length-1].menuProperties.forcePulldownBehavior) {
							setState(menus.length-1, null, null);
							menus[menus.length-1].clearAllDelayed();
						}
					}
				}
				continue;
			}
			
			if (isMouseOver === false) {
				isMouseOver = true;
				dispatchEvent({type: "rollOver", menu: menu});
			}
			
			if (lastMouseOver != i) {
			
				for (var k:Number = i-1; k >= 0; k--) {
					menus[k].clearAllDelayed();
					if (menus[k].getState(menus[k].spawned.fromItem) != 1) {
						/*if (lastItems[k] != menus[k].spawned.fromItem) {
							menus[k].refreshItem(lastItems[k], 0);
							lastItems[k] = menus[k].spawned.fromItem;
						}
						menus[k].refreshItem(menus[k].spawned.fromItem, 1);*/
						
						setState(k, menus[k].spawned.fromItem, 1);
						// restore last active spawn
						menus[k].$daction = menus[k].spawned.fromItem;
					}
				}
				
				lastMouseOver = i;
				
			}
			
			var j:Number = 0;
			
			while (menu.holder[j]) {
				if (!menu.items[j].separator && (menu.items[j].enabled || menu.items[j].enabled == undefined) && menu.holder[j].hitTest(_root._xmouse, _root._ymouse, true)) {
					setState(i, j, 1);
					if (menu.expanded) {
						if (i != 0 || menu.menuProperties.forcePulldownBehavior) {
							menu.delayedSpawn(j);
						}
						else {
							menu.spawn(j);
						}
					}
					return { menu: i, item: j};
				}
				j++;
			}
			
			// textfield overflowing?
			if (!menu.holder[j] && menu.holder.hitTest(_root._xmouse, _root._ymouse, true)) {
				return;
			}
			
			if (menu.expanded) {
				if (i != 0 || menu.menuProperties.forcePulldownBehavior) {
					menu.delayedSpawn(null);
				}
				else {
					menu.removeSpawn();
				}
			}

			setState(i, null, null);
			
			break;
			
		}
		
		if (!hasFocus && !menus[menus.length-1].expanded) {
			setState(menus.length-1, null, null);
		}
		
	}
	
	// called from items
	function rolledOut(Void):Void {
		if (!menus[menus.length-1].expanded) {
			setState(menus.length-1, null, null);
		}
	}
	
	function closeAll(Void):Void {
		hasFocus = false;
		setState(0, null, null);
		menus[0].removeSpawn();
		lastItems = new Array();
		if (menus[0].menuProperties.forcePulldownBehavior) {
			menus[0].hide();
		}
		else {
			menus[0].expanded = false;
		}
	}

	function onMouseDown(Void):Void {
		if (!enabled) {
			return;
		}
		justCreated = false;
		isMouseDown = true;
		lastSpawned = menus.length-1;
		var ret:Object;
		if (ret = onMouseMove()) {
			menus[ret.menu].spawn(ret.item);
		}
		else if (!menus[0].hitTest(_root._xmouse, _root._ymouse, true)) {
			closeAll();
		}
	}
	
	function onMouseUp(Void):Void {
		if (!enabled) {
			return;
		}
		var ret:Object;
		if (ret = onMouseMove()) {
			var d:Object = menus[ret.menu].items[ret.item];
			if (!d.pulldown) {
				closeAll();
				d.m.apply(d.o, d.a);
				dispatchEvent({type: "itemSelect", item: d, menu: [ret.menu]});
			}
			else {
				if (ret.menu == 0 && lastSpawned && !menus[0].menuProperties.forcePulldownBehavior) {
					menus[ret.menu].expanded = false;
					menus[ret.menu].removeSpawn();
					hasFocus = false;
					onMouseMove();
				}
			}
		}
		else if (!menus[0].hitTest(_root._xmouse, _root._ymouse, true) && !justCreated) {
			closeAll();
		}
	}
	
	function register(p_menu:MenuComponent):Number {
		for (var i:Number = 0; i < menus.length; i++) {
			if (p_menu == menus[i]) {
				return i;
			}
		}
		menus.push(p_menu);
		lastItems.push(null);
		return menus.length - 1;
	}
	
	function remove(p_menu:MenuComponent):Void {
		for (var i:Number = 0; i < menus.length; i++) {
			if (p_menu == menus[i]) {
				menus.splice(i, 1);
				lastItems.splice(i, 1);
			}
		}
	}
	
	function MenuInteractionController() {
		init();
	}
	
	function step(p_menu:Number, p_fwd:Boolean) {
		var menu:MenuComponent = menus[p_menu];
		if (lastItems[p_menu] != null) {
			var j:Number = lastItems[p_menu];
		}
		else {
			var j:Number = p_fwd ? -1 : menu.items.length;
		}
		for (var i:Number = 0; i < menu.items.length /*-1*/ ; i++) {
			j += p_fwd ? 1 : -1;
			if (j >= menu.items.length) {
				j = 0;
			}
			else if (j < 0) {
				j = menu.items.length-1;
			}
			if (menu.items[j].separator || (!menu.items[j].enabled && menu.items[j].enabled != undefined)) {
				continue;
			}
			setState(p_menu, j, 1);
			break;
		}
	}
	
	function get excitated():Boolean {
		return lastItems[0] != null;
	}
	
	function activate(Void):Void {
		if (!enabled) {
			if (activateReenables) {
				enabled = true;
			}
			else {
				return;
			}
		}
		if (!hasFocus) {
			hasFocus = true;
			setState(menus.length-1, null, null);
			step(menus.length-1, true);
		}
	}
	
	function onKeyDown(Void):Void {
		
		if (disableKeyboard) {
			return;
		}
		
		if (!hasFocus) {
			if (menus.length-1 == 0 && menus[0].menuProperties.forcePulldownBehavior) {
				hasFocus = true;
			}
			else {
				if (Key.getCode() == activatorKey) {
					activate();
				}
				return;
			}
		}
		
		if (!enabled) {
			return;
		}
		
		if (Key.getCode() == activatorKey && !menus[0].menuProperties.forcePulldownBehavior) {
			closeAll();
		}
		else if ((Key.getCode() == Key.LEFT || Key.getCode() == Key.RIGHT) && (menus.length == 1 && !menus[0].menuProperties.vertical)) {
			step(menus.length-1, Key.getCode() == Key.DOWN || Key.getCode() == Key.RIGHT);
			if (menus[menus.length-1].expanded) {
				menus[menus.length-1].spawn(lastItems[menus.length-1]);
				step(menus.length-1, true);
			}
		}
		else if ((Key.getCode() == Key.DOWN || Key.getCode() == Key.UP) && (menus.length > 1 || menus[0].menuProperties.vertical)) {
			step(menus.length-1, Key.getCode() == Key.DOWN || Key.getCode() == Key.RIGHT);
		}
		else if (Key.getCode() == Key.RIGHT && menus[menus.length-1].menuProperties.vertical) {
			if (lastItems[menus.length-1] == null) {
				step(menus.length-1, true);
				if (lastItems[menus.length-1] != null) {
					return;
				}
			}
			if (menus[menus.length-1].items[lastItems[menus.length-1]].pulldown) {
				menus[menus.length-1].spawn(lastItems[menus.length-1]);
				step(menus.length-1, true);
			}
			else {
				if (!menus[0].menuProperties.vertical) {
					menus[0].removeSpawn();
					step(0, true);
					menus[0].spawn(lastItems[0]);
					step(1, true);
				}
			}
		}
		else if (Key.getCode() == Key.LEFT && menus.length > 1 && menus[menus.length-1].menuProperties.vertical) {
			menus[menus.length-2].removeSpawn();
			if (menus.length == 1 && !menus[0].menuProperties.vertical) {
				step(0, false);
				menus[0].spawn(lastItems[0]);
				step(1, true);
			}
		}
		else if (Key.getCode() == Key.ESCAPE) {
			if (menus.length > 1) {
				if (menus.length == 2) {
					menus[0].expanded = false;
				}
				menus[menus.length-2].removeSpawn();
			}
			else if (menus[0].expanded && !menus[0].menuProperties.forcePulldownBehavior && !menus.length == 1) {
					menus[0].expanded = false;
					menus[0].refreshItem(lastItems[0], 1);
			}
			else {
				closeAll();
			}
			
		}
		else if ((Key.getCode() == Key.UP || Key.getCode() == Key.DOWN) && !menus[menus.length-1].menuProperties.vertical) {
			if (lastItems[menus.length-1] == null) {
				step(menus.length-1, true);
			}
			else if (menus[menus.length-1].items[lastItems[menus.length-1]].pulldown) {
				menus[menus.length-1].spawn(lastItems[menus.length-1]);
				step(menus.length-1, true);
			}
		}
		else if (Key.getCode() == Key.ENTER || ((Key.getCode() == Key.UP || Key.getCode() == Key.DOWN) && !menus[menus.length-1].menuProperties.vertical)) {
			if (lastItems[menus.length-1] == null) {
				step(menus.length-1, true);
			}
			else if (menus[menus.length-1].items[lastItems[menus.length-1]].pulldown) {
				menus[menus.length-1].spawn(lastItems[menus.length-1]);
				step(menus.length-1, true);
			}
			else {
				var d:Object = menus[menus.length-1].items[lastItems[menus.length-1]];
				d.m.apply(d.o, d.a);
				closeAll();
			}
		}
	}
	
	function init(Void):Void {
		dispatch.initialize(this);
		
		menus = new Array();
		lastItems = new Array();
		
		Key.addListener(this);
		Mouse.addListener(this);
	}

	// dispatch
		function dispatchEvent(p_eventObj:Object):Void {}
		function addEventListener(p_eventName:String, p_handler):Void {}
		function addEventListenerL(p_eventName:String, p_obj:Object, p_handler):Void {}
		function removeEventListener(p_eventName:String, p_handler):Void {}
		function removeEventListenerL(p_eventName:String, p_obj:Object, p_handler):Void {}
		function removeAllEventListeners(p_eventName:String):Void {}
			
}